// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


/*
   Establece un manejador externo para controlar el teclado, recoge los parametros de la operacion
   por este y envia la peticion al servidor que devolvera el resultado de aplicar la operacion 
   sobre los parametros recogidos.

   Realiza las peticiones mediante un servicio de reparto por rangos [42, 43]=[+,*], 
   [45, 45]=[-,-] y la division no tiene ningun rango asociado asi que deberia usar el 
   ultimo rango que haya sido establecido.

   El cliente de esta aplicacion es: server.p rserver.p => Transporte: comm::Transport.
*/
#include <iostream>

#include <string.h>

#include <anna/core/core.hpp>
#include <anna/comm/comm.hpp>

#include <anna/test/Menu.hpp>
#include <anna/test/Request.hpp>
#include <anna/test/Response.hpp>

class MyCommunicator : public Communicator {
public:   
   MyCommunicator () : Communicator () {;}
   
private:
   test::Response a_response;   
   test::Request a_request;
   
   void eventReceiveMessage (ClientSocket&, const Message&) throw (RuntimeException);
   void eventUser (const char* id, const void* context) throw ();   
};

typedef comm::ByRangeDelivery<int> MyService;

class BRKClient : public anna::comm::Application {
public:

   BRKClient ();
      
   MyService* getService () const throw () { return a_service; }
   const test::Menu& getMenu () const throw () { return a_menu; }
   
private:
   MyCommunicator a_communicator;
   test::Menu a_menu;
   MyService* a_service;

   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);    
};

using namespace std;
using namespace test;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   BRKClient app;
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("kclient", new TraceWriter ("file.trace", 4096000));
 
      app.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

BRKClient::BRKClient () : 
   Application ("kclient", "BRKClient", "1.0.0"),
   a_menu (&a_communicator)
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Direccion en el que el servidor atiende respuestas.");
   commandLine.add ("px", CommandLine::Argument::Mandatory, "Puertos en los que hay servidores de + y *");
   commandLine.add ("pm", CommandLine::Argument::Mandatory, "Puertos en los que hay servidores de -");
}

void BRKClient::initialize () 
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());    

   Network& network = Network::instantiate ();

   Tokenizer ports;
   int port;
   MyService::range_iterator rr;
   
   const char* ip = cl.getValue ("a");

   a_service = new MyService ("Service_Arithmetic", true);

   rr = a_service->createRange ('*', '+');
   ports.apply (cl.getValue ("px"), ",");
   for (Tokenizer::const_iterator ii = ports.begin (), maxii = ports.end (); ii != maxii; ii ++) {
      port = atoi (Tokenizer::data (ii));
      a_service->attach (rr, network.createServer (ip, port, true));
   }

   rr  = a_service->createRange ('-', '-');
   ports.apply (cl.getValue ("pm"), ",");
   for (Tokenizer::const_iterator ii = ports.begin (), maxii = ports.end (); ii != maxii; ii ++) {
      port = atoi (Tokenizer::data (ii));
      a_service->attach (rr, network.createServer (ip, port, true));
   }

   a_communicator.attach (a_service);
   a_communicator.attach (&a_menu);
}

void BRKClient::run ()
   throw (RuntimeException)
{   
   a_menu.paint ();
   a_communicator.accept ();
}

void MyCommunicator::eventReceiveMessage (ClientSocket&, const Message& message)
   throw (RuntimeException)
{
   a_response.decode (message.getBody ());

   const Millisecond responseTime = anna::functions::millisecond () - a_response.initTime;
   
   cout << endl << "ResponseTime: " << responseTime << " ms" << endl;
   cout << "Resultado de la peticion: " << a_response.x << (char) a_response.op << a_response.y  << " = " << a_response.result << endl << endl;
   
   static_cast <BRKClient&> (anna::comm::functions::getApp ()).getMenu ().paint ();
}

//-----------------------------------------------------------------------------------------
// Cuando el Menu tiene disponibles todos los datos necesarios para la peticin se lo
// notifica al comunicador mediante un evento de usuario.
//-----------------------------------------------------------------------------------------
void MyCommunicator::eventUser (const char* id, const void* context) 
   throw ()
{
   LOGMETHOD (TraceMethod tm ("MyCommunicator", "eventUser", ANNA_FILE_LOCATION));

   if (anna_strcmp (id, Menu::EventData) == 0) {
      const Menu::Data* data (reinterpret_cast <const Menu::Data*>  (context));
      
      a_request.op = data->a_operation;
      a_request.x = data->a_op1;
      a_request.y = data->a_op2;
      a_request.initTime = anna::functions::millisecond ();
      
      MyService* service = static_cast <BRKClient&> (anna::comm::functions::getApp ()).getService ();
      
      try {
         service->prepare (data->a_operation);
         service->send (a_request);
      }
      catch (RuntimeException& ex) {
         ex.trace ();
      }
   } 
}
