// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <functional>

#include <anna/core/core.hpp>
#include <anna/xml/xml.hpp>

#include <anna/app/Application.hpp>

class Test : public app::Application {
public:
   Test ();
   
private:
   void run () throw (anna::RuntimeException);      
};

using namespace std;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;

   xml::functions::initialize ();
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug);
      Logger::initialize ("XML", new TraceWriter ("file.trace", 2048000));

      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}


Test::Test () : 
   app::Application ("testfunctions", "Checking trace system", "1.0") 
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("document", CommandLine::Argument::Mandatory, "XML document");
   commandLine.add ("dtd", CommandLine::Argument::Optional, "DTD file");
}

void Test::run () 
   throw (RuntimeException)
{
   CommandLine& commandLine (CommandLine::instantiate ());

   xml::DocumentFile xmlDoc;
   xml::Parser parser;
   xml::Compiler xmlCompiler;
   xml::DTDFile xmlDTD;
   bool withDTD (false);

   xmlDoc.initialize (commandLine.getValue ("document"));

   if (commandLine.exists ("dtd")) {
      xmlDTD.initialize (commandLine.getValue ("dtd"));
      withDTD = true;
   }

   LOGDEBUG (
       string msg ("DTD: ");
       msg += (withDTD == true) ? "true": "false";
       msg += " | Document: ";
       msg +=  anna::functions::asString (xmlDoc.getContent (), 24);
       Logger::debug (msg, ANNA_FILE_LOCATION);
   );
       
   const xml::Node* root = (withDTD) ? parser.apply (xmlDoc, xmlDTD):  parser.apply (xmlDoc);
   const xml::Node* node;

   cout << "Root: " << root->asString () << endl;

   for (xml::Node::const_child_iterator ii = root->child_begin (), maxii = root->child_end (); ii != maxii; ii ++) {
      node = xml::Node::node (ii);
      cout << "Level 1 - Node: " << node->asString () << endl;
   } 

   cout << "xml::Compiler (Default= xml::Compiler::Mode::Visual): " << endl << xmlCompiler.apply (root) << endl << endl;

   xml::Node* root2 = new xml::Node ("sample");
   xml::Node* node2;
   node2 = root2->createChild ("theNode");
   node2->createAttribute ("attr1", 1222);
   node2->createText ("text");
   node2->createAttribute ("Empty", (char*) NULL);
   delete root2;

   cout << "xml::Compiler2: " << endl << xmlCompiler.apply (root2) << endl << endl;

   const char* compact =  xmlCompiler.apply (xmlDoc, root, xml::Compiler::Mode::Compact); 
   cout << "xml::Compiler (Compact): " << endl << compact << endl << endl;

   xml::DocumentMemory memoDoc;

   DataBlock doc (compact, anna_strlen (compact), false);

   memoDoc.initialize (doc);
   root = (withDTD) ? parser.apply (memoDoc, xmlDTD):  parser.apply (memoDoc);
   for (xml::Node::const_child_iterator ii = root->child_begin (), maxii = root->child_end (); ii != maxii; ii ++) {
      node = xml::Node::node (ii);
      cout << "Level 1 - Memo-Node: " << node->asString () << endl;
   } 
   cout << "xml::CompilerMemo: " << endl << xmlCompiler.apply (root) << endl << endl;

   memoDoc.initialize (compact);
}
