// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_oam_Counter_hpp
#define anna_core_oam_Counter_hpp

#include <anna/core/util/defines.hpp>

namespace anna {

namespace oam {

class CounterScope;

/**
   Contador.

   Permite un sistema rapido para registrar situaciones ocurridas en las aplicaciones.
*/
class Counter {
public:
  typedef unsigned int type_t;

  /**
     Devuelve el nombre logico del contador, que coincidira con el indicado a la hora
     de crear el contador mediante el metodo CounterScope::create.
     \return El nombre logico del contador.
  */
  const std::string& getName() const throw() { return a_name; }

  /**
     Devuelve la referencia logica de este contador. Sera el numero resultante de combinar
     el numero de ambito en el que esta registrado y el numero de contador dentro de este
     ambito.
     \return La referencia logica de este contador.
  */
  int getReference() const throw();

  /**
   * Devuelve el valor actual de este contador. Este valor se podr a cero cada vez que los contadores
   * se graben.
   * \return El valor actual de este contador.
   */
  type_t getValue() const throw() { return a_value; }

  /**
   * Devuelve el valor acumulado de este contador.
   * \return El valor acumulado de este contador.
   */
  U64 getAccumulatedValue() const throw() { return a_accValue; }

  /**
     Operador de conversion.
     \return El valor asociado a este contador.
  */
  operator type_t () const throw() { return a_value; }

  /**
     Inicializa el valor de este contador.
  */
  void reset() throw() { a_value = 0; }

  /**
     Inicializa el valor acumulado de este contador.
     \return Devuelve 'true' si fue reseteado, 'false' si ya lo estaba
  */
  bool resetAcc() throw() { bool result = (a_accValue != 0); a_accValue = 0; return result; }


  /**
     Devuelve una cadena con la informacion relevante de este objeto.
     \return Una cadena con la informacion relevante de este objeto.
  */
  std::string asString() const throw();

private:
  CounterScope& a_scope;
  const int a_id;
  std::string a_name;
  type_t a_value;
  U64 a_accValue;

  Counter(CounterScope& scope, const int id, const char* name);
  Counter(const Counter&);

  void debug() const throw();

  friend class CounterScope;
};

}
}

#endif

