// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_oam_CounterRecorder_hpp
#define anna_core_oam_CounterRecorder_hpp

#include <string>

#include <anna/core/RuntimeException.hpp>

namespace anna {

namespace oam {

class Counter;

/**
   Interfaz que deben implementar los objetos encargados de transferir los contadores de una
   aplicacion a un medio fisico.

   Esta clase se invocara automaticamente al invocar al metodo record() de un modulo oam.
*/
class CounterRecorder {
public:
  /**
     Inicia el proceso de grabacion.
  */
  virtual void open() throw(RuntimeException) = 0;

  /**
     Transfiere el contador recibido como parametro al medio fisico. Solo nos llegaran los
     contadores modificados desde la ultima vez que se ejecuto el proceso de grabacion.
     \param counter Contador que debemos transferir al medio fisico.
  */
  virtual void apply(const Counter& counter) throw(RuntimeException) = 0;

  /**
     Termina el proceso de grabacion.
  */
  virtual void close() throw() = 0;

  /**
     Devuelve una cadena con la informacion relevante de este objeto.
     \return Una cadena con la informacion relevante de este objeto.
  */
  virtual std::string asString() const throw() = 0;
};

}
}

#endif

