// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_Microsecond_hpp
#define anna_core_util_Microsecond_hpp

#include <string>

#include <anna/core/util/defines.hpp>
#include <anna/core/RuntimeException.hpp>

namespace anna {

class Millisecond;
class Second;

class Microsecond {
public:
  typedef S64 type_t;

  /**
   * Constructor
   */
  Microsecond() : a_value(0) {;}

  /**
   * Constructor.
   * \param value Valor inicial de esta instancia.
   */
  explicit Microsecond(const type_t value) : a_value(value) {;}

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Microsecond(const Microsecond& other) : a_value(other.a_value) {;}

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Microsecond(const Millisecond& other);

  /**
   * Constructor copia.
   * \param other Instancia de la que copiar.
   */
  Microsecond(const Second& other);

  /**
   * Conversor a numrico.
   * \return El valor asociado a esta instancia.
   */
  operator type_t () const throw() { return a_value; }

  /**
   * \internal
   */
  type_t& refValue() throw() { return a_value; }

  Microsecond& operator= (const Microsecond& other) throw() { a_value = other.a_value; return *this; }

  Microsecond& operator= (const Millisecond& other) throw();

  Microsecond& operator= (const Second& other) throw();

  bool operator== (const Microsecond& other) const throw() { return a_value == other.a_value; }

  bool operator== (const Millisecond& other) const throw();

  bool operator== (const Second& other) const throw();

  bool operator!= (const Microsecond& other) const throw() { return a_value != other.a_value; }

  bool operator!= (const Millisecond& other) const throw();

  bool operator!= (const Second& other) const throw();

  bool operator> (const Microsecond& other) const throw() { return a_value > other.a_value; }

  bool operator> (const Millisecond& other) const throw();

  bool operator> (const Second& other) const throw();

  bool operator< (const Microsecond& other) const throw() { return a_value < other.a_value; }

  bool operator< (const Millisecond& other) const throw();

  bool operator< (const Second& other) const throw();

  bool operator>= (const Microsecond& other) const throw() { return a_value >= other.a_value; }

  bool operator>= (const Millisecond& other) const throw() { return (operator==(other) == true) ? true : operator>(other); }

  bool operator>= (const Second& other) const throw() { return (operator==(other) == true) ? true : operator>(other); }

  bool operator<= (const Microsecond& other) const throw() { return a_value <= other.a_value; }

  bool operator<= (const Millisecond& other) const throw() { return (operator==(other) == true) ? true : operator<(other); }

  bool operator<= (const Second& other) const throw() { return (operator==(other) == true) ? true : operator<(other); }

  Microsecond& operator+= (const Microsecond& other) throw() { a_value += other.a_value; return *this; }
  Microsecond& operator*= (const int& value) throw() { a_value *= value; return *this; }

  /**
   * Devuelve el valor asociado a esta instancia.
   * \return el valor asociado a esta instancia.
   */
  type_t getValue() const throw() { return a_value; }

  /**
   * Devuelve la hora actual de sistema expresada en microsegundos transcurridos desde el 1 de Enero de 1970
   * \return la hora actual de sistema expresada en microsegundos transcurridos desde el 1 de Enero de 1970
   */
  static Microsecond getTime() throw();

  /**
   * Devuelve una cadena con el valor de esta instancia y las unidades "us".
   * \return una cadena con el valor de esta instancia y las unidades "us".
   */
  std::string asString() const throw();

  /**
   * Obtiene los microsegundos del valor contenido en la cadena recibida como parmetro.
   * \param value Cadena que contiene los microsegundos habr sido obtenida con #asString.
   * \return los microsegundos del valor contenido en la cadena recibida como parmetro.
   */
  static Microsecond fromString(const std::string& value) throw(RuntimeException);

private:
  type_t a_value;

  friend class Millisecond;
  friend class Second;

  friend class Microsecond operator - (const Microsecond& left, const Microsecond& right) throw();
  friend class Microsecond operator + (const Microsecond& left, const Microsecond& right) throw();
  friend class Microsecond operator / (const Microsecond& left, const Microsecond& right) throw();
  friend class Microsecond operator / (const Microsecond& left, const int right) throw();
  friend class Microsecond operator / (const Microsecond& left, const unsigned int right) throw();
};

inline Microsecond operator - (const Microsecond& left, const Microsecond& right)
throw() {
  return Microsecond(left.a_value - right.a_value);
}

inline Microsecond operator + (const Microsecond& left, const Microsecond& right)
throw() {
  return Microsecond(left.a_value + right.a_value);
}

inline Microsecond operator / (const Microsecond& left, const Microsecond& right)
throw() {
  return Microsecond(left.a_value / right.a_value);
}

inline Microsecond operator / (const Microsecond& left, const int right)
throw() {
  return Microsecond(left.a_value / right);
}

inline Microsecond operator / (const Microsecond& left, const unsigned int right)
throw() {
  return Microsecond(left.a_value / right);
}

}

#endif
