// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_MultiRangeExpression_hpp
#define anna_core_util_MultiRangeExpression_hpp


// STL
#include <string>
#include <map>


namespace anna {

/**
* Class helper to manage multi-range expression like '1-4,23,45-46' (1,2,3,4,23,45,46)
*/
class MultiRangeExpression {

  std::string a_literal;
  std::map < unsigned int, int/*dummy*/ > a_data; // expands literal


  void refresh(void) throw();  // keep coherence between 'a_data' and 'a_literal'


public:

  MultiRangeExpression() {};
  ~MultiRangeExpression() {};


  /**
  * Gets the configured literal by mean #setLiteral or #addLiteral
  *
  * @return Literal
  */
  const char * getLiteral(void) const throw() { return a_literal.c_str(); }

  /**
  * Gets expanded representation for stored literal. E.g.: '1-3,8,10' => '1,2,3,7,8,10'
  *
  * @return Expanded literal
  */
  std::string getExpandedLiteral(void) const throw();

  /**
  * Simplify stored literal. E.g.: '1,1,1,2,3,7,8,10' => '1-3,8,10' and returns it.
  *
  * @return Simplified literal
  */
  const char * simplifyLiteral(void) throw();

  // helpers

  /**
  * Returns true if the value provided is contained in the multirange expression literal
  *
  * @param value Value to be tested
  * @return True or false
  */
  bool contain(const unsigned int & value) const throw() { return (a_data.find(value) != a_data.end()); }

  // setters

  /**
  * Configures a new literal
  *
  * @param l Literal to be stored
  */
  void setLiteral(const char * l) throw() {
    a_literal = l ? l : "";
    refresh();
  }

  /**
  * Accumulates the provided literal over the stored literal
  * You could simplify with #simplifyLiteral, because perhaps there is overlapping between current literal and provided one.
  *
  * @param l Literal to be added
  */
  void addLiteral(const char * l) throw() {
    if(!l) return;

    if(std::string(l) != "") {
      a_literal += ",";
      a_literal += l;
    }

    refresh();
  }
};

};


#endif
