// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_app_dcca_Message_hpp
#define anna_diameter_app_dcca_Message_hpp


// Local
#include <anna/diameter/app/base/Message.hpp>

// AVPID's
#include <anna/diameter/helpers/base/defines.hpp>
#include <anna/diameter/helpers/tgpp/defines.hpp>
#include <anna/diameter/helpers/dcca/defines.hpp>

#include <anna/core/RuntimeException.hpp>

// STL
#include <string>


//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------



namespace anna {

namespace diameter {

namespace codec {
using namespace basetypes;
}

namespace app {

namespace dcca {



typedef struct {

  std::string qosProfile;
  int Release;
  int TrafficClass;
  int HandlingPriority;
  int MaximumBitrateForUplink;
  int MaximumBitrateForDownlink;


  void reset() throw() {
    qosProfile = "";
    Release = TrafficClass = HandlingPriority = MaximumBitrateForUplink = MaximumBitrateForDownlink = 0;
  }

} qosProfile_t;




/**
* Diameter message for DCCA (diameter credit control application)
*/
class Message : public anna::diameter::app::base::Message {

  qosProfile_t a_qosProfile;

public:

  /**
     Avp Subscription-Id-Data data-part pointer reference.

     @param Subscription-Id-Type value to filter the search. END_USER_E164 by default.
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * getSubscriptionIdData(int subscriptionIdType = helpers::dcca::AVPVALUES__Subscription_Id_Type::END_USER_E164, anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      const anna::diameter::codec::Avp * sid;
      int pos = 1;

      while((sid = getAvp(helpers::dcca::AVPID__Subscription_Id, pos++)))
      if(subscriptionIdType == sid->getAvp(helpers::dcca::AVPID__Subscription_Id_Type)->getEnumerated()->getValue())
        return sid->getAvp(helpers::dcca::AVPID__Subscription_Id_Data)->getUTF8String();
        return NULL;
      );
  }

  /**
     Avp Multiple-Services-Credit-Control pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getMultipleServicesCreditControl(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control));
  }

  /**
     Avp Service-Identifier data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * getServiceIdentifier(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->getAvp(helpers::dcca::AVPID__Service_Identifier)->getUnsigned32());
  }

  /**
     Avp Rating-Group data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * getRatingGroup(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->getAvp(helpers::dcca::AVPID__Rating_Group)->getUnsigned32());
  }

  /**
     Avp Tariff-Change-Usage data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getTariffChangeUsage(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::dcca::AVPID__Used_Service_Unit)->
               getAvp(helpers::dcca::AVPID__Tariff_Change_Usage)->getEnumerated();
    );
  }

  /**
     Avp Validity-Time data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * getValidityTime(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->getAvp(helpers::dcca::AVPID__Validity_Time)->getUnsigned32());
  }

  /**
     Avp Requested-Service-Unit pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getRequestedServiceUnit(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->getAvp(helpers::dcca::AVPID__Requested_Service_Unit));
  }

  /**
     Avp n-th Used-Service-Unit pointer reference
     @param ocurrence Select the n-th USU within the MSCC. First by default.
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getUsedServiceUnit(int ocurrence = 1, anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::dcca::AVPID__Used_Service_Unit, ocurrence);
    );
  }

  /**
     Avp CC_Request_Type data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getCCRequestType(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__CC_Request_Type)->getEnumerated());
  }

  /**
     Avp CC_Request_Number data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * getCCRequestNumber(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__CC_Request_Number)->getUnsigned32());
  }

  /**
     Avp Multiple-Services-Indicator data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getMultipleServicesIndicator(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Indicator)->getEnumerated());
  }

  /**
     Avp Service-Context-Id data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * getServiceContextId(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Service_Context_Id)->getUTF8String());
  }

  /**
     Avp 3GPP-NSAPI data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * get3GPPNSAPI(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::tgpp::AVPID__3GPP_NSAPI)->getUTF8String());
  }

  /**
     Avp 3GPP-Selection-Mode data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * get3GPPSelectionMode(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::tgpp::AVPID__3GPP_Selection_Mode)->getUTF8String());
  }

  /**
     Avp 3GPP-Session-Stop-Indicator data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const OctetString * get3GPPSessionStopIndicator(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::tgpp::AVPID__3GPP_Session_Stop_Indicator)->getOctetString());
  }

  /**
     Avp User-Equipment-Info pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getUserEquipmentInfo(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__User_Equipment_Info));
  }

  /**
     Avp User-Equipment-Info-Value data-part pointer reference.

     @param User-Equipment-Info-Type value to filter the search. IMEISV by default.
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const OctetString * getUserEquipmentInfoValue(int userEquipmentInfoType = helpers::dcca::AVPVALUES__User_Equipment_Info_Type::IMEISV, anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      const anna::diameter::codec::Avp * uei;
      int pos = 1;

      while((uei = getAvp(helpers::dcca::AVPID__User_Equipment_Info, pos++)))
      if(userEquipmentInfoType == uei->getAvp(helpers::dcca::AVPID__User_Equipment_Info_Type)->getEnumerated()->getValue())
        return uei->getAvp(helpers::dcca::AVPID__User_Equipment_Info_Value)->getOctetString();
        return NULL;
      );
  }

  // there has to be at least one out-of-line definition of a non-pure-virtual function (http://www.daniweb.com/software-development/cpp/threads/114299/undefined-reference-to-vtable-for-)
  // Lo que dice es que las virtuales no puras de una clase deben tener al menos una definicion en el .cc (lo contrario de in-line). Parece una limitacin del compilador.
  // Podemos hacer una definicion por defecto (return NULL), o mejor, hacerla virtual pura (porque se necesitara por parte del decodificador de QoS):
  virtual const UTF8String * get3GPPGPRSNegQoSProfile(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) = 0;


  /**
     Decodes 3GPP-GPRS-Neg-QoS-Profile (as described in 3GPP TS 24.008)

     @return 3GPP-GPRS-Neg-QoS-Profile decoded on struct
  */
  const qosProfile_t & decode3GPPGPRSNegQoSProfile() throw(anna::RuntimeException);



//   static bool dameTiempoConsumido(const InfoDIAMETER & mensaje,long &tiempoConsumidoAntesConmutacion,long &tiempoConsumidoDespuesConmutacion,long &tiempoConsumidoSinConmutacion);
//   static bool dameTiempoConsumido(const InfoDIAMETER & mensaje,long &tiempoConsumidoAntesConmutacion,long &tiempoConsumidoDespuesConmutacion,long &tiempoConsumidoSinConmutacion,bool &hayUSUantes, bool &hayUSUdespues);
//
//   // nos devuelve el volumen que viene en el mensaje diameter (antes, despues de la conmutacion y el valor cuando no ha  habido conmutacion)
//   // Le pasamos el tipo volumen que queremos  ( ambos,downlink,uplink)
//   // en tipoVol, se indica de que   AVP leemos las unidades ( Input_Octets, Output-Octets o TotalOctets )
//   //static void dameVolumConsumido(const InfoDIAMETER & mensaje,int tipoVol,long &volAntesConmutacion,long &volDespuesConmutacion,long &volConsumidoSinConmutacion);
//   // Devuelve FALSE si ha habido algun error
//   static bool dameVolumConsumido(const InfoDIAMETER & mensaje,int tipoVol,long &volUpAntesConmutacion,long &volUpDespuesConmutacion,
//                  long &volDownAntesConmutacion,long &volDownDespuesConmutacion,long &volUp,long &volDown,bool &errorTariffChange);
//   static bool dameVolumConsumido(const InfoDIAMETER & mensaje,int tipoVol,long &volUpAntesConmutacion,long &volUpDespuesConmutacion,
//                  long &volDownAntesConmutacion,long &volDownDespuesConmutacion,long &volUp,long &volDown,bool &errorTariffChange,
//                  bool &hayUSUantes, bool &hayUSUdespues);
//
//
//   bool getConsumedTime(int &beforeSwitch, int &afterSwitch, int &withoutSwitch) const throw();
//   bool getConsumedVolume(int volumeType, int &upBeforeSwitch, int &upAfterSwitch, int &downBeforeSwitch, int &downAfterSwitch, int &upWithoutSwitch, int &downWithoutSwitch, bool &tariffChangeError) const throw();
//





};

}
}
}
}

#endif
