// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_app_dcca_huawei_Message_hpp
#define anna_diameter_app_dcca_huawei_Message_hpp


// Local
#include <anna/diameter/app/dcca/Message.hpp>

// AVPID's
#include <anna/diameter/helpers/huawei/defines.hpp>

#include <anna/core/RuntimeException.hpp>


//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------



namespace anna {

namespace diameter {

namespace codec {
using namespace basetypes;
}

namespace app {

namespace dcca {

namespace huawei {


/**
* Diameter message for Nokia DCCA (diameter credit control application)
*/
class Message : public anna::diameter::app::dcca::Message {

public:

  /**
     Avp PS-Information pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getPSInformation(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information);
    );
  }

  /**
     Avp Called-Station-Id data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * getCalledStationId(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::radius::AVPID__Called_Station_Id)->getUTF8String();
    );
  }

  /**
     Avp 3GPP-PDP-Type data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * get3GPPPDPType(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_PDP_Type)->getUnsigned32();
    );
  }

  /**
     Avp 3GPP-SGSN-MCC-MNC data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * get3GPPSGSNMCCMNC(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_SGSN_MCC_MNC)->getUTF8String();
    );
  }

  /**
     Avp 3GPP-Charging-Characteristics data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * get3GPPChargingCharacteristics(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_Charging_Characteristics)->getUTF8String();
    );
  }

  /**
     Avp 3GPP-GPRS-Neg-QoS-Profile data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const UTF8String * get3GPPGPRSNegQoSProfile(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_GPRS_Neg_QoS_Profile)->getUTF8String();
    );
  }

  /**
     Avp CG-Address data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Address * getCGAddress(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__CG_Address)->getAddress();
    );
  }

  /**
     Avp 3GPP-Rat-Type data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const OctetString * get3GPPRatType(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_Rat_Type)->getOctetString();
    );
  }

  /**
     Avp SGSN-Address data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Address * getSGSNAddress(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__SGSN_Address)->getAddress();
    );
  }

  /**
     Avp GGSN-Address data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Address * getGGSNAddress(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__GGSN_Address)->getAddress();
    );
  }

  /**
     Avp 3GPP-Charging-Id data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Unsigned32 * get3GPPChargingId(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__3GPP_Charging_Id)->getUnsigned32();
    );
  }

  /**
     Avp Reporting-Reason data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getReportingReason(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(return getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->getAvp(helpers::tgpp::AVPID__Reporting_Reason)->getEnumerated());
  }

  /**
     Avp Trigger-Type data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getTriggerType(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::tgpp::AVPID__Trigger)->
               getAvp(helpers::tgpp::AVPID__Trigger_Type)->getEnumerated();
    );
  }

  /**
     Avp PS-Furnish-Charging-Information pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const anna::diameter::codec::Avp * getPSFurnishChargingInformation(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::tgpp::AVPID__PS_Furnish_Charging_Information);
    );
  }

  /**
     Avp PS-Free-Format-Data data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const OctetString * getPSFreeFormatData(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::tgpp::AVPID__PS_Furnish_Charging_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Free_Format_Data)->getOctetString();
    );
  }

  /**
     Avp PS-Append-Free-Format-Data data-part pointer reference
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Enumerated * getPSAppendFreeFormatData(anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::dcca::AVPID__Multiple_Services_Credit_Control)->
               getAvp(helpers::tgpp::AVPID__PS_Furnish_Charging_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Append_Free_Format_Data)->getEnumerated();
    );
  }

  /**
     Avp n-th PDP-Address pointer reference
     @param ocurrence Select the n-th PDP-Address within the PS-Information. First by default.
     @param emode Excepcion mode handling when avp is not found: Ignore (no action is taken but debug trace), Throw (exception launched, by default), Trace (trace warning).
  */
  const Address * getPDPAddress(int ocurrence = 1, anna::Exception::Mode::_v emode = anna::Exception::Mode::Throw) throw(anna::RuntimeException) {
    ASSERT_APP_GETAVP(
      return   getAvp(helpers::tgpp::AVPID__Service_Information)->
               getAvp(helpers::tgpp::AVPID__PS_Information)->
               getAvp(helpers::tgpp::AVPID__PDP_Address)->getAddress();
    );
  }
};

}
}
}
}
}

#endif
