// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_codec_Engine_hpp
#define anna_diameter_codec_Engine_hpp


// STL
#include <string>

#include <anna/core/util/Recycler.hpp>

#include <anna/diameter/codec/Message.hpp>
#include <anna/diameter/codec/Avp.hpp>
#include <anna/diameter/codec/EngineImpl.hpp>


using namespace anna::diameter::codec;


//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------


namespace anna {

namespace diameter {

namespace codec {



/**
 * Standard inheritance for engine component implementation, allocating basic Avp and Message classes.
 */
class Engine : public EngineImpl {

public:

  /**
     Constructor
     @param className Logical name for the class.
  */
  Engine(const char *className, const stack::Dictionary * dictionary) : EngineImpl(className, dictionary) {;}

  void releaseAvp(Avp* avp) throw() {
    if(avp == NULL) return;

    //Avp* aux = static_cast <Avp*> (avp);
//        avp/*aux*/->clear(); // free internal data-part storage specially for grouped avps which will release its childrens
    a_avps.release(avp/*aux*/);
  }

  void releaseMessage(Message* message) throw() {
    if(message == NULL) return;

    //Message* aux = static_cast <Message*> (message);
//        message/*aux*/->clear(); // free internal data-part storage specially for childrens releasing
    a_messages.release(message/*aux*/);
  }

protected:

  anna::Recycler<Avp> a_avps;
  anna::Recycler<Message> a_messages;

  anna::diameter::codec::Avp* allocateAvp() throw() { return a_avps.create(); }
  anna::diameter::codec::Message* allocateMessage() throw() { return a_messages.create(); }

  friend class Message;
  friend class Avp;
};

}
}
}

#endif

