// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_helpers_base_functions_hpp
#define anna_diameter_helpers_base_functions_hpp


// Local
#include <anna/diameter/defines.hpp>
#include <anna/config/defines.hpp>

#include <anna/core/RuntimeException.hpp>

// STL
#include <string>

//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------

namespace anna {
class DataBlock;
}


namespace anna {

namespace diameter {

namespace helpers {

namespace base {


struct functions {

  // getters & helpers

  /**
  * Gets the first level command Result-Code from datablock provided
  *
  * @param db Provided Message DataBlock
  *
  * @return Result-Code value
  */
  static U32 getResultCode(const anna::DataBlock &) throw(anna::RuntimeException);

  /**
  * Gets the tokenized by ';' Session-Id contents defined as '<DiameterIdentity>;<high 32 bits>;<low 32 bits>[;<optional value>]':
  *
  * @param sessionId Provided UTF8String Session-Id content
  * @param diameterIdentity Sender Diameter Identity
  * @param high High 64-bit-sequence part
  * @param low Low 64-bit-sequence part
  * @param optional Empty string when missing
  */
  static void decodeSessionId(const std::string &sessionId, std::string &diameterIdentity, U32 &high, U32 &low, std::string &optional) throw(anna::RuntimeException);

  /**
  * Gets the fixed Session-Id AVP (UTF8String) from datablock provided.
  * Also gets the tokenized by ';' Session-Id contents defined as '<DiameterIdentity>;<high 32 bits>;<low 32 bits>[;<optional value>]':
  *
  * @param db Provided Message DataBlock
  * @param diameterIdentity Sender Diameter Identity
  * @param high High 64-bit-sequence part
  * @param low Low 64-bit-sequence part
  * @param optional Empty string when missing
  *
  * @return Session-Id
  */
  static std::string getSessionId(const anna::DataBlock &db, std::string &diameterIdentity, U32 &high, U32 &low, std::string &optional) throw(anna::RuntimeException);


  /**
  * Gets the fixed Session-Id AVP (UTF8String) from datablock provided.
  *
  * @param db Provided Message DataBlock
  *
  * @return Session-Id
  */
  static std::string getSessionId(const anna::DataBlock &db) throw(anna::RuntimeException) {
    std::string dummy1, dummy4;
    U32 dummy2, dummy3;
    return (getSessionId(db, dummy1, dummy2, dummy3, dummy4));
  }
};


}
}
}
}


#endif

