// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_xml_Data_hpp
#define anna_xml_Data_hpp

#include <string>
#include <stdlib.h>

namespace anna {

namespace xml {

class Node;

/**
   Dato generico de nodo XML.

   Cada nodo XML puede tener una serie indeterminada de dato XMLs. Por ejemplo:

   \code
      <broadcast>
         <INetAddress Address="204.152.65.15" Port="2000"/>
         <INetAddress Address="204.152.65.47" Port="2002"/>
      </broadcast>
      <Indicador>
      Es es un texto libre
      </Indicador>
   \endcode

   Cada uno de los nodos \em INetAddress tiene dos dato XMLs (\em Address y \em Port).
*/
class Data {
public:
  /**
     Devuelve el valor asociado a este dato XML.
     \return El valor asociado a este dato XML.
  */
  const std::string& getValue() const throw() { return a_value; }

  /**
     Devuelve el valor asociado a este dato XML.
     \return El valor asociado a este dato XML.
  */
  const char* getCStringValue() const throw() { return a_value.c_str(); }

  /**
     Devuelve el valor numerico asociado a este dato XML.
     \return El valor numerico asociado a este dato XML.
  */
  int getIntegerValue() const throw() { return atoi(a_value.c_str()); }

  /**
   * Establece el valor de ste dato XML.
   * \param value Valor que tomar ste dato XML.
   */
  void setValue(const char* value) throw() { a_value = (value != NULL) ? value : ""; filter(a_value); }

  /**
   * Establece el valor de ste dato XML.
   * \param value Valor que tomar ste dato XML.
   */
  void setValue(const std::string& value) throw() { a_value = value; }

  /**
   * Establece el valor de ste dato XML.
   * \param value Valor que tomar ste dato XML.
   */
  void setValue(const int value) throw();

  /**
     Devuelve una cadena con toda la informacion relevante de esta instancia.
     \return Una cadena con toda la informacion relevante de esta instancia.
  */
  virtual std::string asString() const throw();

protected:
  /**
   * Contructor.
   */
  Data() : a_owner(NULL) {;}

private:
  std::string a_value;
  Node* a_owner;

  void setNode(Node* node) throw() { a_owner = node; }

  /**
     Filtra la cadena recibida como parametro para asegurar que no contiene ninguna
     expresion o caracter que pudiera causar un error al establecer el xml::Data con
     este valor.
     \return La instancia de la cadena una vez modificada.
  */
  static const std::string& filter(const std::string& str) throw() { return str; }

  friend class Node;
};

}
}

#endif
