// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdlib.h>
#include <time.h>
#include <netinet/in.h>

#include <anna/core/RuntimeException.hpp>
#include <anna/config/defines.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/util/Second.hpp>
#include <anna/core/util/Millisecond.hpp>
#include <anna/core/util/Microsecond.hpp>

#include <anna/comm/CompatCodec.hpp>
#include <anna/comm/functions.hpp>
#include <anna/comm/Variable.hpp>

using namespace std;
using namespace anna;

bool comm::CompatCodec::st_initScramble(false);

// static
template <class T, class M> comm::Variable* insert(const char* name, const short int id, M& theVector, T& value)
throw(RuntimeException) {
  comm::Variable* result = theVector.find(id);

  if(result != NULL)
    throw RuntimeException(functions::asString("Variable Id %d alreay used", id), ANNA_FILE_LOCATION);

  theVector.add(result = new comm::Variable(id, name, value));
  LOGDEBUG(
    String msg("insert <T, M> | ");
    msg <<  result->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return result;
}

// static
template <class T, class M> comm::Variable* insertRef(const char* name, const short int id, M& theVector, T& value)
throw(RuntimeException) {
  comm::Variable* result = theVector.find(id);

  if(result != NULL)
    throw RuntimeException(functions::asString("Variable Id %d alreay used", id), ANNA_FILE_LOCATION);

  theVector.add(result = new comm::Variable(id, name, value.refValue()));
  LOGDEBUG(
    String msg("insert <T, M> | ");
    msg <<  result->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return result;
}

comm::CompatCodec::CompatCodec(const comm::CompatCodec::Type type, const bool scramble)  :
  comm::Message(comm::Message::StatusCodeBuffer::None),
  a_type(type),
  a_scramble(scramble),
  a_nullCounter(0) {
  if(a_scramble == true && st_initScramble == false) {
    st_initScramble = true;
    srand(time(NULL));
  }
}

comm::CompatCodec::~CompatCodec() {
  for(iterator ii = begin(), maxii = end(); ii != maxii; ii ++)
    delete variable(ii);

  a_variables.clear();
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, std::string& value)
throw(RuntimeException) {
  return insert(name, id, a_variables, value);
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, int& value)
throw(RuntimeException) {
  const int backup(value);
  const Variable* result = insert(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, S64& value)
throw(RuntimeException) {
  const S64 backup(value);
  const Variable* result = insert(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, bool& value)
throw(RuntimeException) {
  const bool backup(value);
  const Variable* result = insert(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, DataBlock& value)
throw(RuntimeException) {
  return insert(name, id, a_variables, value);
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, float& value)
throw(RuntimeException) {
  const float backup(value);
  const Variable* result = insert(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, double& value)
throw(RuntimeException) {
  const double backup(value);
  const Variable* result = insert(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, Second& value)
throw(RuntimeException) {
  const Second backup(value);
  const Variable* result = insertRef(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, Millisecond& value)
throw(RuntimeException) {
  const Millisecond backup(value);
  const Variable* result = insertRef(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, Microsecond& value)
throw(RuntimeException) {
  const Microsecond backup(value);
  const Variable* result = insertRef(name, id, a_variables, value);
  value = backup;
  return result;
}

const comm::Variable* comm::CompatCodec::attach(const char* name, const short int id, comm::CompatCodec& value)
throw(RuntimeException) {
  if(&value == this) {
    String msg("comm::CompatCodec::attach | Variable: ");
    msg << name << " | Can not link with itself";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return insert(name, id, a_variables, value);
}

void comm::CompatCodec::setNull(const short int id, const bool isNull)
throw(RuntimeException) {
  Variable* variable = a_variables.find(id);

  if(variable == NULL)
    throw RuntimeException(functions::asString("Id %d is not defined", id), ANNA_FILE_LOCATION);

  if(variable->isNull()) {
    if(isNull == false) a_nullCounter --;
  }
  else {
    if(isNull == true) a_nullCounter ++;
  }

  variable->setNull(isNull);
}

void comm::CompatCodec::setNull(const comm::Variable* variable, const bool isNull)
throw() {
  if(variable->isNull()) {
    if(isNull == false) a_nullCounter --;
  }
  else {
    if(isNull == true) a_nullCounter ++;
  }

  const_cast <comm::Variable*>(variable)->setNull(isNull);
}

bool comm::CompatCodec::isNull(const short int id) const
throw(RuntimeException) {
  const Variable* variable = a_variables.find(id);

  if(variable == NULL)
    throw RuntimeException(functions::asString("Id %d is not defined", id), ANNA_FILE_LOCATION);

  return variable->isNull();
}

const comm::Variable& comm::CompatCodec::find(const short int id) const
throw(RuntimeException) {
  const Variable* variable = a_variables.find(id);

  if(variable == NULL)
    throw RuntimeException(functions::asString("Id %d is not defined", id), ANNA_FILE_LOCATION);

  return *variable;
}

//-----------------------------------------------------------------------
// Codificacin de los datos en un bloque de memoria.
//
// mensaje ::= <byte0><byte1><byte2> <dato> ... <dato>
//
// dato ::= <byteA><byteB><byteC> [<cadena | int | long | bloque>]
// cadena ::= <byteLSB><byteMSB> < contenido > </0>
// int ::= <byteLSB> <byte> <byte> <byteMSB>
// long ::= <byteLSB> <byte> <byte> <byteMSB>
// bloque ::= <int> < contenido >
//
// (0) byte0 = Si es distinto de cero => contiene la clave usada para codificar
//     el mensaje.
// (2) byte1 = Nmero de datos que contiene el mensaje.
//     Recordar que slo se transfieren a este bloque de memoria los datos que
//     tengan asignado algn value.
// (1) byte2 = Typeentificador del mensaje interno.
// (3) Los bytes A y B contienen del identificador del dato.
// (4) Los cuatro primeros bits del byteC indican el tipo de dato. En los
//     tipos de datos boolean su contenido se guarda en el bit mas significativo
//     de este byte.
// (5) En las cadenas los dos primeros byres indican la longitud de la cadena
//     incluyendo el indicador de fin de cadena, luego aparece el contenido
//     de la cadena y finalmente el 0. El cero se incluye para optimizar
//     la recogida de datos.
//-----------------------------------------------------------------------
const DataBlock& comm::CompatCodec::code()
throw(RuntimeException) {
  unsigned char c(0);
  iterator ii;
  iterator maxii(a_variables.end());
  Variable* variable;
  int stringLen;
  const char* string;
  char aux [sizeof(S64)];

  if(a_scramble == true)     // (1)
    while(c == 0) c = rand() % 0xff;

  DataBlock& self = *this;
  self.clear();
  self += c;
  self += (char)(a_variables.size() - a_nullCounter);          // (2)
  self += comm::CompatCodec::getType();                              // (1)

  for(ii = begin(); ii != maxii; ii ++) {
    variable = CompatCodec::variable(ii);

    if(a_nullCounter > 0 && variable->isNull() == true)
      continue;

    switch(variable->getType()) {
    case Variable::Type::String:
      self += variable->codec();
      stringLen = anna_strlen(string = variable->getStringValue()) + 1;
      self.append(comm::functions::codeShort(aux, stringLen), sizeof(short int));
      self.append(string, stringLen);
      break;
    case Variable::Type::Integer:
      self += variable->codec();
      self.append(comm::functions::codeInteger(aux, variable->getInteger()), sizeof(int));
      break;
    case Variable::Type::Integer64:
      self += variable->codec();
      self.append(comm::functions::codeInteger64(aux, variable->getInteger64()), sizeof(S64));
      break;
    case Variable::Type::Boolean:
      self.append(comm::functions::codeShort(aux, variable->getId()), sizeof(short int));      // (3)
      c = Variable::Type::Boolean;

      if(variable->getBoolean() == true)    // (4)
        c |= 0x80;

      self += c;
      break;
    case Variable::Type::Block:
      self += variable->codec();
      self.append(comm::functions::codeInteger(aux, variable->getDataBlock().getSize()), sizeof(int));
      self += variable->getDataBlock();
      break;
    case Variable::Type::Float:
      self += variable->codec();
      self.append(comm::functions::codeFloat(aux, variable->getFloat()), sizeof(float));
      break;
    case Variable::Type::Double:
      self += variable->codec();
      self.append(comm::functions::codeDouble(aux, variable->getDouble()), sizeof(float));
      break;
    case Variable::Type::Custom:
      self += variable->codec();
      {
        const DataBlock& codec = reinterpret_cast <CompatCodec*>(variable->getCustom())->code();
        self.append(comm::functions::codeInteger(aux, codec.getSize()), sizeof(int));
        self += codec;
      }
      break;
    default: break;
    }
  }

  LOGDEBUG(

    // Se sacan aqui para evitar qtene que comprobar el estado de las trazas en cada paso
    for(ii = begin(); ii != maxii; ii ++)
    Logger::debug(CompatCodec::variable(ii)->asString(), ANNA_FILE_LOCATION);
    Logger::write(Logger::Debug, "comm::CompatCodec::code", self, ANNA_FILE_LOCATION)
  );

  if(a_scramble == true) {
    char* data = const_cast <char*>(self.getData());
    int size = self.getSize();

    for(int i = 1, key = data [0]; i < size; i ++)
      data [i] ^= key ++;
  }

  return self;
}

//-------------------------------------------------------------------------------------------
void comm::CompatCodec::decode(const DataBlock& dataBlock)
throw(RuntimeException) {
  const char* data = dataBlock.getData();
  const int size = dataBlock.getSize();

  if(size < 1)
    throw RuntimeException("Can not decode an empty DataBlock", ANNA_FILE_LOCATION);

  if(data [0] != 0)
    for(int i = 1, key(data [0]); i < size; i ++)
      const_cast <char*>(data)[i] ^= key ++;

  LOGDEBUG(Logger::write(Logger::Debug, "comm::CompatCodec::decode", dataBlock, ANNA_FILE_LOCATION));
  const int maxdata = data [1];

  if(maxdata != a_variables.size())
    normalDecode(data, size, maxdata);
  else if(optimizedDecode(data, size) == false)
    normalDecode(data, size, maxdata);

  LOGDEBUG(

    for(const_iterator ii = begin(), maxii = end(); ii != maxii; ii ++)
    Logger::debug(CompatCodec::variable(ii)->asString(), ANNA_FILE_LOCATION);
  );
}

//-------------------------------------------------------------------------------------------
// Decodifica los buffers que pueden contener variables nulas.
//-------------------------------------------------------------------------------------------
void comm::CompatCodec::normalDecode(const char* data, const int size, const int maxdata)
throw(RuntimeException) {
  // Mientras no se demuestre lo contrario todas las variables son nulas
  a_nullCounter = a_variables.size();

  for(iterator ii = begin(), maxii = end(); ii != maxii; ii ++)
    CompatCodec::variable(ii)->setNull(true);

  short int id;
  int nbytes;
  Variable* variable;
  bool hasError = false;
  const char* top = data + size;
  data += 3;

  for(int ndata = 0; ndata < maxdata && data < top; ndata ++) {
    id = comm::functions::decodeShort(data);
    data += sizeof(short int);

    if((variable = a_variables.find(id)) == NULL) {
      hasError = true;
      break;
    }

    a_nullCounter --;

    switch(*data  & 0x7f) {
    case Variable::Type::String:
      nbytes = comm::functions::decodeShort(++ data);
      variable->setValue(data += sizeof(short int));
      data += nbytes;
      break;
    case Variable::Type::Integer:
      variable->setInteger(comm::functions::decodeInteger(++ data));
      data += sizeof(int);
      break;
    case Variable::Type::Integer64:
      variable->setValue(comm::functions::decodeInteger64(++ data));
      data += sizeof(S64);
      break;
    case  Variable::Type::Boolean:
      variable->setBoolean((*data & 0x80) ? true : false);
      data ++;
      break;
    case Variable::Type::Block:
      nbytes = comm::functions::decodeInteger(++ data);
      variable->setDataBlock(DataBlock(data += sizeof(int), nbytes, false));
      data += nbytes;
      break;
    case Variable::Type::Float:
      variable->setFloat(comm::functions::decodeFloat(++ data));
      data += sizeof(float);
      break;
    case Variable::Type::Double:
      variable->setDouble(comm::functions::decodeDouble(++ data));
      data += sizeof(double);
      break;
    case Variable::Type::Custom:
      nbytes = comm::functions::decodeInteger(++ data);
      {
        DataBlock dataBlock(data += sizeof(int), nbytes, false);
        reinterpret_cast <CompatCodec*>(variable->getCustom())->decode(dataBlock);
      }
      data += nbytes;
      break;
    }
  }

  if(hasError == true) {
    string msg("comm::CompatCodec::normalDecode | Buffer: ");
    msg += functions::asString(DataBlock(data, size, false));
    msg += functions::asText(" | Id is not defined: ", id);
    Logger::error(msg, ANNA_FILE_LOCATION);
  }

  if(a_nullCounter < 0) {
    a_nullCounter = 0;
    LOGWARNING(
      string msg("comm::CompatCodec::normalDecode | Buffer: ");
      msg += functions::asString(DataBlock(data, size, false));
      msg += " | NullCounter is less than zero";
      Logger::warning(msg, ANNA_FILE_LOCATION);
    );
  }
}

//-------------------------------------------------------------------------------------------
// Decodifica optimizadamente buffers que no tengan variables nulas.
// (1) El codigo de la variable
//-------------------------------------------------------------------------------------------
bool comm::CompatCodec::optimizedDecode(const char* data, const int size)
throw(RuntimeException) {
  int nbytes;
  Variable* variable;
  const char* top = data + size;
  data += 3;

  for(iterator ii = begin(), maxii = end(); ii != maxii && data < top; ii ++) {
    data += sizeof(short int);
    variable = CompatCodec::variable(ii);

    switch(*data & 0x7f) {
    case Variable::Type::String:
      nbytes = comm::functions::decodeShort(++ data);
      variable->setValue(data += sizeof(short int));
      data += nbytes;
      break;
    case Variable::Type::Integer:
      variable->setInteger(comm::functions::decodeInteger(++ data));
      data += sizeof(int);
      break;
    case Variable::Type::Integer64:
      variable->setValue(comm::functions::decodeInteger64(++ data));
      data += sizeof(S64);
      break;
    case  Variable::Type::Boolean:
      variable->setBoolean((*data & 0x80) ? true : false);
      data ++;
      break;
    case Variable::Type::Block:
      nbytes = comm::functions::decodeInteger(++ data);
      variable->setDataBlock(DataBlock(data += sizeof(int), nbytes, false));
      data += nbytes;
      break;
    case Variable::Type::Float:
      variable->setFloat(comm::functions::decodeFloat(++ data));
      data += sizeof(float);
      break;
    case Variable::Type::Double:
      variable->setDouble(comm::functions::decodeDouble(++ data));
      data += sizeof(double);
      break;
    case Variable::Type::Custom:
      nbytes = comm::functions::decodeInteger(++ data);
      {
        DataBlock dataBlock(data += sizeof(int), nbytes, false);
        reinterpret_cast <CompatCodec*>(variable->getCustom())->decode(dataBlock);
      }
      data += nbytes;
      break;
    }
  }

  return (data == top);
}

comm::CompatCodec::Type comm::CompatCodec::getType(const DataBlock& dataBlock)
throw(RuntimeException) {
  const int size(dataBlock.getSize());

  if(size <= 1)
    throw RuntimeException("DataBlock is not valid", ANNA_FILE_LOCATION);

  const char* data(dataBlock.getData());
  comm::CompatCodec::Type result;
  result = (data [0] != 0) ? ((data [0] + 1) ^ data [2]) : data [2];
  LOGDEBUG(Logger::write(Logger::Debug, "comm::CompatCodec::getType", result, ANNA_FILE_LOCATION));
  return result;
}

comm::CompatCodec::VariableContainer::VariableContainer() {
  a_maxSize = 16;
  a_size = 0;
  a_variables = new Variable* [a_maxSize];
  anna_memset(a_variables, 0, sizeof(Variable*) * a_maxSize);
}

void comm::CompatCodec::VariableContainer::add(comm::Variable* variable)
throw() {
  if(a_size == a_maxSize) {
    int maxSize = (a_maxSize << 1) - (a_maxSize >> 1);
    Variable** variables = new Variable* [maxSize];
    anna_memset(variables, 0, sizeof(Variable*) * maxSize);
    anna_memcpy(variables, a_variables, sizeof(Variable*) * a_size);
    delete a_variables;
    a_variables = variables;
    a_maxSize = maxSize;
  }

  a_variables [a_size ++] = variable;
}

comm::Variable* comm::CompatCodec::VariableContainer::find(const int id)
throw() {
  for(int ii = 0; ii < a_size; ii ++) {
    if(a_variables [ii]->getId() == id)
      return a_variables [ii];
  }

  return NULL;
}

const comm::Variable* comm::CompatCodec::VariableContainer::find(const int id) const
throw() {
  for(int ii = 0; ii < a_size; ii ++) {
    if(a_variables [ii]->getId() == id)
      return a_variables [ii];
  }

  return NULL;
}

void comm::CompatCodec::VariableContainer::clear()
throw() {
  delete [] a_variables;
  a_maxSize = a_size = 0;
}

