// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdio.h>

#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/util/Tokenizer.hpp>
#include <anna/core/util/defines.hpp>

#include <anna/core/util/TextComposer.hpp>
#include <anna/core/util/TextVariable.hpp>

using namespace std;
using namespace anna;

//---------------------------------------------------------------------------------------
// (1) Cada uno de los DynamicText se encarga de liberar la memoria de sus componentes.
//---------------------------------------------------------------------------------------
TextComposer::~TextComposer() {
  for(variable_iterator ii = begin(), maxii = end(); ii != maxii; ii ++)
    delete textVariable(ii);

  container::clear();
  delete a_prefix;
}

/*
 * Este mtodo se invoca desde el TextManager::create.
 * Divide la expressin en partes ms fciles de analizar.
 *
 * La expresin puede tener la forma "texto inicial ${variable1:%s} siguiente texto ${variable2:%02d} el resto"
 *
 * Y quedar como:
 *    a_prefix = "texto inicial "
 *    var1 => nombre = variable1 => "%s siguiente texto"
 *    var2 => nombre = variable2 => "%02d el resto"
 */
void TextComposer::initialize()
throw(RuntimeException) {
  Tokenizer variables(a_expression, "${");
  Tokenizer idAndText;
  Tokenizer nameAndFormat;
  String expression;
  Variable::Type::_v type;
  TextVariable* variable;

  // Si el texto no tiene ninguna variable
  if(variables.size() == 1) {
    a_prefix = new String(a_expression);
    return;
  }

  try {
    bool init = true;

    for(Tokenizer::const_iterator ii = variables.begin(), maxii = variables.end(); ii != maxii; ii ++) {
      if(anna_strchr(Tokenizer::data(ii), '}') == NULL) {
        if(init == true) {
          a_prefix = new String(Tokenizer::data(ii));
          init = false;
          continue;
        } else
          throw RuntimeException("Parenthesis are not balanced", ANNA_FILE_LOCATION);
      }

      // El id [0] contendr el <name>:<format>
      // El id [1] contendr el texto (si es que hay).
      idAndText.apply(Tokenizer::data(ii), "}");
      nameAndFormat.apply(idAndText [0], ":");

      if(find(nameAndFormat [0], Exception::Mode::Ignore) != NULL) {
        String msg("Variable ");
        msg << nameAndFormat [0] << " already defined";
        throw RuntimeException(msg, ANNA_FILE_LOCATION);
      }

      type = calculeType(nameAndFormat [1]);
      expression = nameAndFormat [1];

      // Es posible que no haya texto que acompae a identificador
      if(idAndText.size() > 1)
        expression += idAndText [1];

      push_back(variable = new TextVariable(nameAndFormat [0], type, expression));
      LOGDEBUG(
        String msg("TextComposer::initialize | New Variable: ");
        msg << variable->asString();
        Logger::debug(msg, ANNA_FILE_LOCATION)
      );
      init = false;
    }
  } catch(RuntimeException& ex) {
    String msg(asString());
    msg << " | " << ex.getText();
    throw RuntimeException(msg, ex.getFromFile(), ex.getFromLine());
  }
}

TextVariable* TextComposer::find(const char* name, const Exception::Mode::_v emode)
throw(RuntimeException) {
  for(variable_iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
    if(anna_strcmp(textVariable(ii)->getName(), name) == 0)
      return textVariable(ii);
  }

  if(emode == Exception::Mode::Ignore)
    return NULL;

  String msg(asString());
  msg << " | TextVariable: " << name << " | It was not found";

  if(emode == Exception::Mode::Throw)
    throw RuntimeException(msg, ANNA_FILE_LOCATION);

  Logger::error(msg, ANNA_FILE_LOCATION);
  return NULL;
}

String TextComposer::apply() const
throw(RuntimeException) {
  LOGMETHOD(TraceMethod ttmm(Logger::Local7, "TextComposer::apply", ANNA_FILE_LOCATION));
  String result;
// No hace falta proteger porque slo se podr acceder desde el Protector
//   Guard <TextComposer> guard (this);

  if(a_prefix != NULL)
    result = *a_prefix;

  try {
    a_buffer.clear();

    for(const_variable_iterator ii = begin(), maxii = end(); ii != maxii; ii ++)
      result += textVariable(ii)->compose(a_buffer);

    LOGDEBUG(
      String msg("TextComposer { Id: ");
      msg << a_id << " | Text: " << result << " }";
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
  } catch(RuntimeException& ex) {
    String msg(asString());
    msg << " | " << ex.getText();
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return result;
}

String TextComposer::asString() const
throw() {
  String result("TextComposer { Id: ");
  result << a_id;
  result << " | Expression: " << a_expression;
  return result << " }";
}

//static
TextVariable::Type::_v TextComposer::calculeType(const char* format)
throw(RuntimeException) {
  static const char* typeInteger = "[:digit:]*d";
  static const char* typeString = "[:digit:]*s";
  static const char* typeFloat = "([:digit:]*.[:digit:]|.[:digit:]*)f|f";
  static const char* typeInteger64 = "[:digit:]*(ld|lld)";

  if(*format != '%')
    throw RuntimeException("Format must begin with character %%", ANNA_FILE_LOCATION);

  if(functions::isLike(typeInteger, format))
    return Variable::Type::Integer;

  if(functions::isLike(typeString, format))
    return Variable::Type::String;

  if(functions::isLike(typeFloat, format))
    return Variable::Type::Float;

  if(functions::isLike(typeInteger64, format))
    return Variable::Type::Integer64;

  String msg("Format '");
  msg << format << "' is not recognized";
  throw RuntimeException(msg, ANNA_FILE_LOCATION);
}

