// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/mt/Guard.hpp>

#include <anna/core/util/TextManager.hpp>
#include <anna/core/util/TextComposer.hpp>


using namespace std;
using namespace anna;

TextManager::TextManager(const char* name) :
  a_name(name) {
}

void TextManager::clear()
throw() {
  Guard guard(*this, "TextManager::clear");

  for(TextComposerVector::iterator ii = a_composers.begin(), maxii = a_composers.end(); ii != maxii; ii ++)
    delete *ii;

  a_composers.clear();
}

void TextManager::create(const int composer, const char* expression)
throw(RuntimeException) {
  TextComposer* result = NULL;
  Guard guard(*this, "TextManager::create");

  if((result = xfind(composer)) != NULL) {
    string msg("Id: ");
    msg += functions::asString(composer);
    msg += " | Id already used in ";
    msg += result->asString();
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  try {
    if((result = createTextComposer(composer, expression)) == NULL) {
      string msg(a_name);
      msg += "::createTextComposer must not return NULL";
      throw RuntimeException(msg, ANNA_FILE_LOCATION);
    }

    LOGDEBUG(
      string msg("Compositor created | ");
      msg += result->asString();
      Logger::write(Logger::Debug, msg, ANNA_FILE_LOCATION);
    );
    result->initialize();
    a_composers.push_back(result);
  } catch(RuntimeException&) {
    delete result;
    throw;
  }
}

TextComposer&  TextManager::find(const int composer)
throw(RuntimeException) {
  TextComposer* result;
  Guard guard(*this, "TextManager::find");

  if((result = xfind(composer)) == NULL) {
    string msg("Id: ");
    msg += functions::asString(composer);
    msg += " | Was not created";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return *result;
}

const TextComposer&  TextManager::find(const int composer) const
throw(RuntimeException) {
  const TextComposer& result = const_cast <TextManager*>(this)->find(composer);
  return result;
}

TextComposer* TextManager::createTextComposer(const int composer, const char* expression)
throw() {
  return new TextComposer(composer, expression);
}

TextComposer* TextManager::xfind(const int composer)
throw() {
  for(TextComposerVector::iterator ii = a_composers.begin(), maxii = a_composers.end(); ii != maxii; ii ++)
    if((*ii)->getId() == composer) {
      return (*ii);
    }

  return NULL;
}
