// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter/stack/Dictionary.hpp>
#include <anna/diameter/stack/Engine.hpp>

#include <anna/config/defines.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/xml/xml.hpp>
#include <anna/core/functions.hpp>

#include <anna/config/defines.hpp>
#include <anna/core/functions.hpp>
#include <anna/diameter/codec/Format.hpp>

// STL
#include <string>


using namespace anna::diameter::stack;
using namespace anna;


//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------
#define ITEM_OVERWRITE(item_n,item,pool,poolNames)\
LOGNOTICE(\
   std::string trace = "\n   Updated ";\
   trace += item_n;\
   trace += ":\n";\
   trace += anna::functions::tab(found->asString(), 6);\
   trace += "\n   New content:\n";\
   trace += anna::functions::tab(item.asString(), 6);\
   trace += "\n";\
   anna::Logger::notice(trace, ANNA_FILE_LOCATION);\
);\
std::string name = found->getName();\
pool.erase(pool.find(found->getId()));\
poolNames.erase(poolNames.find(name));


//------------------------------------------------------------------------------
//----------------------------------------------------- Dictionary::Dictionary()
//------------------------------------------------------------------------------
Dictionary::Dictionary(void) {
  a_dtd = (Engine::instantiate()).getDictionariesDTD();
  a_allowUpdates = false;
  initialize();
}

//------------------------------------------------------------------------------
//----------------------------------------------------- Dictionary::initialize()
//------------------------------------------------------------------------------
void Dictionary::initialize() throw() {
  a_formats.clear();
  a_vendors.clear();
  a_avps.clear();
  a_commands.clear();
  a_vendorNames.clear();
  a_avpNames.clear();
  a_commandNames.clear();
  a_dtd = (Engine::instantiate()).getDictionariesDTD();
  // RFC3588 Diameter Formats harcoding:
  // Basic diameter types
  Format OctetString(this), Integer32(this), Integer64(this), Unsigned32(this), Unsigned64(this), Float32(this), Float64(this), Grouped(this);
  OctetString.setRFC3588(codec::Format::OctetString);
  Integer32.setRFC3588(codec::Format::Integer32);
  Integer64.setRFC3588(codec::Format::Integer64);
  Unsigned32.setRFC3588(codec::Format::Unsigned32);
  Unsigned64.setRFC3588(codec::Format::Unsigned64);
  Float32.setRFC3588(codec::Format::Float32);
  Float64.setRFC3588(codec::Format::Float64);
  Grouped.setRFC3588(codec::Format::Grouped);
  addFormat(OctetString);
  addFormat(Integer32);
  addFormat(Integer64);
  addFormat(Unsigned32);
  addFormat(Unsigned64);
  addFormat(Float32);
  addFormat(Float64);
  addFormat(Grouped);
  // Derived diameter types
  Format Address(this), Time(this), UTF8String(this), DiameterIdentity(this), DiameterURI(this), Enumerated(this), IPFilterRule(this), QoSFilterRule(this);
  Address.setRFC3588(codec::Format::Address);                     Address.setParentName(OctetString.getName());
  Time.setRFC3588(codec::Format::Time);                           Time.setParentName(OctetString.getName());
  UTF8String.setRFC3588(codec::Format::UTF8String);               UTF8String.setParentName(OctetString.getName());
  DiameterIdentity.setRFC3588(codec::Format::DiameterIdentity);   DiameterIdentity.setParentName(OctetString.getName());
  DiameterURI.setRFC3588(codec::Format::DiameterURI);             DiameterURI.setParentName(OctetString.getName());
  Enumerated.setRFC3588(codec::Format::Enumerated);               Enumerated.setParentName(Integer32.getName());
  IPFilterRule.setRFC3588(codec::Format::IPFilterRule);           IPFilterRule.setParentName(OctetString.getName());
  QoSFilterRule.setRFC3588(codec::Format::QoSFilterRule);         QoSFilterRule.setParentName(OctetString.getName());
  addFormat(Address);
  addFormat(Time);
  addFormat(UTF8String);
  addFormat(DiameterIdentity);
  addFormat(DiameterURI);
  addFormat(Enumerated);
  addFormat(IPFilterRule);
  addFormat(QoSFilterRule);
  // Generic AVP hardcoding:
  Avp genericAvp(this);
  genericAvp.setCode(0);
  genericAvp.setVendorId(0/*Vendor::Code::Ietf*/);
  genericAvp.setName("AVP");
  Format Any(this);
  Any.setRFC3588(codec::Format::Any);
  addFormat(Any, true /*reserved*/);
  genericAvp.setFormatName(Any.getName());
  genericAvp.setVbit(Avp::FlagRule::mustnot);
  genericAvp.setMbit(Avp::FlagRule::may);
  genericAvp.setPbit(Avp::FlagRule::may);
  genericAvp.setMayEncrypt(false);
  addAvp(genericAvp);
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Dictionary::addFormat()
//------------------------------------------------------------------------------
void Dictionary::addFormat(const Format & format, bool reserved) throw(anna::RuntimeException) {
  if(!reserved && format.isReserved()) {
    std::string s_ex = anna::functions::asString("Format type '%s' is reserved for internal use", format.getName().c_str());
    throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
  }

  const Format * found = getFormat(format.getName());
  if(found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add a format with an existing type name: ";
      //s_ex += format.getName();
      s_ex += format.asString();
      s_ex += "'";
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    LOGNOTICE(
      std::string trace = "\n   Updated format:\n";
      trace += anna::functions::tab(found->asString(), 6);
      trace += "\n   New content:\n";
      trace += anna::functions::tab(format.asString(), 6);
      trace += "\n";
      anna::Logger::notice(trace, ANNA_FILE_LOCATION);
    );
  }

  a_formats[format.getName()] = format;
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Dictionary::addVendor()
//------------------------------------------------------------------------------
void Dictionary::addVendor(const Vendor & vendor) throw(anna::RuntimeException) {

  const Vendor * found = getVendor(vendor.getId());
  if(found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add a vendor with an existing code: ";
      s_ex += vendor.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("vendor", vendor, a_vendors, a_vendorNames);
  }

  found = getVendor(vendor.getName());
  if(found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add a vendor with an existing name: ";
      s_ex += vendor.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("vendor", vendor, a_vendors, a_vendorNames);
  }

  a_vendors[vendor.getId()] = vendor;
  a_vendorNames[vendor.getName()] = getVendor(vendor.getId());
}


//------------------------------------------------------------------------------
//--------------------------------------------------------- Dictionary::addAvp()
//------------------------------------------------------------------------------
void Dictionary::addAvp(const Avp & avp) throw(anna::RuntimeException) {

  const Avp * found = getAvp(avp.getId());
  if (found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add an avp with an existing identifier (code,vendor):\n";
      s_ex += avp.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("avp", avp, a_avps, a_avpNames);
  }

  if((found = getAvp(avp.getName()))) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add an avp with an existing name:\n";
      s_ex += avp.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("avp", avp, a_avps, a_avpNames);
  }

  a_avps[avp.getId()] = avp;
  a_avpNames[avp.getName()] = getAvp(avp.getId());
}


//------------------------------------------------------------------------------
//----------------------------------------------------- Dictionary::addCommand()
//------------------------------------------------------------------------------
void Dictionary::addCommand(const Command & command) throw(anna::RuntimeException) {
  
  const Command * found = getCommand(command.getId());
  if(found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add a command with an existing identifier:\n";
      s_ex += command.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("command", command, a_commands, a_commandNames);
  }

  
  found = getCommand(command.getName());
  if(found) {
    if(!a_allowUpdates) {
      std::string s_ex = "Cannot add a command with an existing name:\n";
      s_ex += command.asString();
      throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
    }

    ITEM_OVERWRITE("command", command, a_commands, a_commandNames);
  }

  a_commands[command.getId()] = command;
  a_commandNames[command.getName()] = getCommand(command.getId());
}


// public

//------------------------------------------------------------------------------
//------------------------------------------------------ Dictionary::getFormat()
//------------------------------------------------------------------------------
const Format * Dictionary::getFormat(const std::string & formatName) const throw() {
  const_format_iterator it = a_formats.find(formatName);

  if(it != format_end()) return ((const Format *) & ((*it).second));

  return (NULL);
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Dictionary::getVendor()
//------------------------------------------------------------------------------
const Vendor * Dictionary::getVendor(S32 vendorId) const throw() {
  const_vendor_iterator it = a_vendors.find(vendorId);

  if(it != vendor_end()) return ((const Vendor *) & ((*it).second));

  return (NULL);
}


//------------------------------------------------------------------------------
//------------------------------------------------------ Dictionary::getVendor()
//------------------------------------------------------------------------------
const Vendor * Dictionary::getVendor(const std::string & vendorName) const throw() {
  const_vendorNames_iterator v_it = a_vendorNames.find(vendorName);

  if(v_it != a_vendorNames.end()) return ((*v_it).second);

  return (NULL);
}


//------------------------------------------------------------------------------
//--------------------------------------------------------- Dictionary::getAvp()
//------------------------------------------------------------------------------
const Avp * Dictionary::getAvp(const AvpId & avpId) const throw() {
  const_avp_iterator it = a_avps.find(avpId);

  if(it != avp_end()) return ((const Avp *) & ((*it).second));

  return (NULL);
}


//------------------------------------------------------------------------------
//--------------------------------------------------------- Dictionary::getAvp()
//------------------------------------------------------------------------------
const Avp * Dictionary::getAvp(const std::string & avpName) const throw() {
  const_avpNames_iterator a_it = a_avpNames.find(avpName);

  if(a_it != a_avpNames.end()) return ((*a_it).second);

  return (NULL);
}


//------------------------------------------------------------------------------
//----------------------------------------------------- Dictionary::getCommand()
//------------------------------------------------------------------------------
const Command * Dictionary::getCommand(const CommandId & commandId) const throw() {
  const_command_iterator it = a_commands.find(commandId);

  if(it != command_end()) return ((const Command *) & ((*it).second));

  return (NULL);
}


//------------------------------------------------------------------------------
//----------------------------------------------------- Dictionary::getCommand()
//------------------------------------------------------------------------------
const Command * Dictionary::getCommand(const std::string & commandName) const throw() {
  const_commandNames_iterator c_it = a_commandNames.find(commandName);

  if(c_it != a_commandNames.end()) return ((*c_it).second);

  return (NULL);
}


//------------------------------------------------------------------------------
//------------------------------------------------------- Dictionary::asString()
//------------------------------------------------------------------------------
std::string Dictionary::asString(void) const throw() {
  std::string trace, title;
  trace += "\n";
  anna::functions::TextHighlightMode::_v thm = anna::functions::TextHighlightMode::LeftAndRightline;
  trace += anna::functions::highlight("Name", thm);
  trace += a_name; trace += "\n";
  const_format_iterator f_it;
  const_vendor_iterator v_it;
  const_avp_iterator a_it;
  const_command_iterator c_it;
  trace += "\n";
  // Formats
  title = "Formats";
  //title = "Formats (";
  //title += anna::functions::entriesAsString(format_size());
  //title += ")";
  trace += anna::functions::highlight(title, thm);

  for(f_it = format_begin(); f_it != format_end(); f_it++) {
    if((*f_it).second.isReserved()) continue;

    if((*f_it).second.isRFC3588())
      trace += "(Diameter RFC-3588 type)    ";
    else
      trace += "(Application-specific type) ";

    trace += (*f_it).second.asString();
    trace += "\n";
  }

  trace += "\n";
  // Vendors
  title = "Vendors (";
  title += anna::functions::entriesAsString(vendor_size());
  title += ")";
  trace += anna::functions::highlight(title, thm);

  for(v_it = vendor_begin(); v_it != vendor_end(); v_it++) {
    trace += (*v_it).second.asString();
    trace += "\n";
  }

  trace += "\n";
  // Avps
  title = "Avps (";
  title += anna::functions::entriesAsString(avp_size());
  title += ")";
  trace += anna::functions::highlight(title, thm);

  for(a_it = avp_begin(); a_it != avp_end(); a_it++) {
    trace += (*a_it).second.asString();
    trace += "\n";
  }

  trace += "\n";
  // Commands
  title = "Commands (";
  title += anna::functions::entriesAsString(command_size());
  title += ")";
  trace += anna::functions::highlight(title, thm);

  for(c_it = command_begin(); c_it != command_end(); c_it++) {
    trace += (*c_it).second.asString();
    trace += "\n";
  }

  trace += "\n";
  return (trace);
}


//------------------------------------------------------------------------------
//---------------------------------------------------------- Dictionary::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* Dictionary::asXML(anna::xml::Node* parent) const throw() {
//   <!ELEMENT dictionary (format*, vendor*, avp*, command*)>
//   <!ATTLIST dictionary name CDATA #REQUIRED>
  anna::xml::Node* result = parent->createChild("dictionary");
  result->createAttribute("name", a_name);

  // Formats
  for(const_format_iterator it = format_begin(); it != format_end(); it++) {
    if((*it).second.isReserved()) continue;

    if((*it).second.isRFC3588()) continue;

    (*it).second.asXML(result);
  }

  // Vendors
  for(const_vendor_iterator it = vendor_begin(); it != vendor_end(); it++)
    (*it).second.asXML(result);

  // Avps
  for(const_avp_iterator it = avp_begin(); it != avp_end(); it++) {
    if((*it).second.getFormat()->isAny()) continue;  // Generic AVP not shown

    (*it).second.asXML(result);
  }

  // Commands
  for(const_command_iterator it = command_begin(); it != command_end(); it++)
    (*it).second.asXML(result);

  return result;
}



//------------------------------------------------------------------------------
//---------------------------------------------------- Dictionary::asXMLString()
//------------------------------------------------------------------------------
std::string Dictionary::asXMLString() const throw() {
  anna::xml::Node root("root");
  return anna::xml::Compiler().apply(asXML(&root));
}



////------------------------------------------------------------------------------
////----------------------------------------- Dictionary::checkUniqueIdentifiers()
////------------------------------------------------------------------------------
//void Dictionary::checkUniqueIdentifiers(const anna::xml::Node *rootNode) const throw(anna::RuntimeException) {
//   std::map < std::string/*xml ref*/, int/*dummy*/ > formats;
//   std::map < std::string/*xml ref*/, int/*dummy*/ > vendors;
//   std::map < std::string/*xml ref*/, int/*dummy*/ > avps;
//   std::map < std::string/*xml ref*/, int/*dummy*/ > commands;
//   std::string nodeName, ref;
//
//   for (anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
//      nodeName = (*it)->getName();
//
//      if (nodeName == "format") {
//         ref = (*it)->getAttribute("name")->getValue();
//         int refs_i = formats.size(); formats[ref] = 0;
//
//         if (formats.size() == refs_i) {
//            std::string s_ex = anna::functions::asString("Repeated format name reference '%s' not allowed within the same xml dictionary", ref.c_str());
//            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
//         }
//      } else if (nodeName == "vendor") {
//         ref = (*it)->getAttribute("name")->getValue();
//         int refs_i = vendors.size(); vendors[ref] = 0;
//
//         if (vendors.size() == refs_i) {
//            std::string s_ex = anna::functions::asString("Repeated vendor name reference '%s' not allowed within the same xml dictionary", ref.c_str());
//            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
//         }
//      } else if (nodeName == "avp") {
//         ref = (*it)->getAttribute("name")->getValue();
//         int refs_i = avps.size(); avps[ref] = 0;
//
//         if (avps.size() == refs_i) {
//            std::string s_ex = anna::functions::asString("Repeated avp name reference '%s' not allowed within the same xml dictionary", ref.c_str());
//            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
//         }
//      } else if (nodeName == "command") {
//         ref = (*it)->getAttribute("name")->getValue();
//         int refs_i = commands.size(); commands[ref] = 0;
//
//         if (commands.size() == refs_i) {
//            std::string s_ex = anna::functions::asString("Repeated command name reference '%s' not allowed within the same xml dictionary", ref.c_str());
//            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
//         }
//      }
//   }
//}


//------------------------------------------------------------------------------
//------------------------------------------------- Dictionary::extractFormats()
//------------------------------------------------------------------------------
void Dictionary::extractFormats(const anna::xml::Node *rootNode) throw(anna::RuntimeException) {
  Format aux;

  for(anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
    std::string nodeName = (*it)->getName();

    if(nodeName == "format") {
      // Reset:
      aux.initialize(this);
      // Attributes:
      const char * type = (*it)->getAttribute("name")->getCStringValue(); // mandatory
      const char * parentType = (*it)->getAttribute("parent-type")->getCStringValue(); // mandatory
      // Assignments:
      aux.setName(type);
      aux.setParentName(parentType /* never empty, basics registered at initialization */);
      // New entry:
      addFormat(aux);
    }
  }
}


//------------------------------------------------------------------------------
//------------------------------------------------- Dictionary::extractVendors()
//------------------------------------------------------------------------------
void Dictionary::extractVendors(const anna::xml::Node *rootNode) throw(anna::RuntimeException) {
  Vendor aux;

  for(anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
    std::string nodeName = (*it)->getName();

    if(nodeName == "vendor") {
      // Attributes:
      const char * name = (*it)->getAttribute("name")->getCStringValue(); // mandatory
      S32 code = (*it)->getAttribute("code")->getIntegerValue(); // mandatory
      // Assignments:
      aux.setId(code);
      aux.setName(name);
      // New entry:
      addVendor(aux);
    }
  }
}


//------------------------------------------------------------------------------
//---------------------------------------------------- Dictionary::extractAvps()
//------------------------------------------------------------------------------
void Dictionary::extractAvps(const anna::xml::Node *rootNode) throw(anna::RuntimeException) {
  Avp auxAvp;
  const anna::xml::Node *singleNode, *groupedNode;

  for(anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
    std::string nodeName = (*it)->getName();

    if(nodeName == "avp") {
      // Reset:
      auxAvp.initialize(this);
      // Attributes:
      const char * name = (*it)->getAttribute("name")->getCStringValue(); // mandatory
      S32 code = (*it)->getAttribute("code")->getIntegerValue(); // mandatory
      const anna::xml::Attribute *vendor_name = (*it)->getAttribute("vendor-name", anna::Exception::Mode::Ignore); // implied
      S32 vendorCode = 0; /* IETF by default */

      if(vendor_name) {
        const char * c_name = vendor_name->getCStringValue();
        auxAvp.setVendorName(c_name);
        const_vendorNames_iterator v_it = a_vendorNames.find(c_name);

        if(v_it == a_vendorNames.end()) {
          std::string s_ex = anna::functions::asString("Vendor '%s', referenced at '%s' avp definition, not found at xml", c_name, name);
          throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
        }

        vendorCode = ((*v_it).second)->getId();
      }

      // Assignments:
      auxAvp.setCode(code);
      auxAvp.setVendorId(vendorCode);
      auxAvp.setName(name);
      // libxml2 doesn't support default attribute value retrieving. All default values will be replaced by #IMPLIED sintax:
      const anna::xml::Attribute *v_bit, *m_bit, *p_bit, *may_encript;
      v_bit = (*it)->getAttribute("v-bit", anna::Exception::Mode::Ignore);
      m_bit = (*it)->getAttribute("m-bit", anna::Exception::Mode::Ignore);
      p_bit = (*it)->getAttribute("p-bit", anna::Exception::Mode::Ignore);
      may_encript = (*it)->getAttribute("may-encrypt", anna::Exception::Mode::Ignore);
      auxAvp.setVbit(v_bit ? (Avp::FlagRule::asEnum(v_bit->getCStringValue())) : (Avp::FlagRule::mustnot));
      auxAvp.setMbit(m_bit ? (Avp::FlagRule::asEnum(m_bit->getCStringValue())) : (Avp::FlagRule::may));
      auxAvp.setPbit(p_bit ? (Avp::FlagRule::asEnum(p_bit->getCStringValue())) : (Avp::FlagRule::may));
      auxAvp.setMayEncrypt(may_encript ? ((may_encript->getValue()) == "yes") : false);

      // Check vendor specific bit:
      if(vendorCode && (auxAvp.getVbit() == Avp::FlagRule::mustnot)) {
        std::string s_ex = anna::functions::asString("Flag rules for vendor specific bit (mustnot) at '%s' avp definicion, are incompatible with non-zeroed vendor id %s", name, getVendor(vendorCode)->asString().c_str());
        throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
      }

      if(!vendorCode && (auxAvp.getVbit() == Avp::FlagRule::must)) {
        std::string s_ex = anna::functions::asString("Flag rules for vendor specific bit (must) at '%s' avp definicion, are incompatible with zeroed vendor id %s", name, getVendor(vendorCode)->asString().c_str());
        throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
      }

      // Single or grouped:
      singleNode = (*it)->find("single", anna::Exception::Mode::Ignore);
      groupedNode = (*it)->find("grouped", anna::Exception::Mode::Ignore);

      if(singleNode) {
        // Attributes:
        const char * formatName = singleNode->getAttribute("format-name")->getCStringValue(); // mandatory
        const anna::xml::Attribute *_enum = singleNode->getAttribute("enum", anna::Exception::Mode::Ignore); // implied
        // Assignments:
        const Format *format = getFormat(formatName);

        if(!format) {
          std::string s_ex = anna::functions::asString("Format '%s', referenced at '%s' avp definition, not found at dictionary (neither xml nor RFC 3588 diameter format types)", formatName, name);
          throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
        }

        auxAvp.setFormatName(formatName);

        if(_enum) {
          const char *enums = _enum->getCStringValue();

          if(!format->isEnumerated()) {
            std::string s_ex = anna::functions::asString("Enumerated literal '%s' is not allowed for '%s' avp format", enums, formatName);
            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
          }

          auxAvp.setEnums(enums);
        }

        for(anna::xml::Node::const_child_iterator l_it = singleNode->child_begin(); l_it != singleNode->child_end(); l_it++) {
          // Attributes:
          std::string data = (*l_it)->getAttribute("data")->getValue(); // mandatory
          std::string alias = (*l_it)->getAttribute("alias")->getValue(); // mandatory
          // Assignment:
          auxAvp.addLabel(data, alias);
        }
      } else { // grouped
        // Assignments:
        auxAvp.setFormatName(codec::Format::asText(codec::Format::Grouped));
        // Wait for avprule insertion, because we need complete avp reference pool (*)
      }

      // New entry:
      addAvp(auxAvp);
    } // end if
  } // end for

  // (*) Avp rules adding:
  for(anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
    std::string nodeName = (*it)->getName();

    if(nodeName == "avp") {
      // Attributes:
      const char * gid = (*it)->getAttribute("name")->getCStringValue(); // mandatory
      // Avp:
      const_avpNames_iterator a_it = a_avpNames.find(gid);
      Avp * gavp = (Avp *)((*a_it).second);

      if(!gavp) continue;  // it could be mising (a redefinition could have removed it)

      const Format *format = gavp->getFormat();

      // Avprule updating:
      if(format->isGrouped()) {
        AvpRule auxAvpRule(this);
        groupedNode = (*it)->find("grouped");

        for(anna::xml::Node::const_child_iterator r_it = groupedNode->child_begin(); r_it != groupedNode->child_end(); r_it++) {
          // Attributes:
          std::string id = (*r_it)->getAttribute("id")->getValue(); // mandatory
          std::string type = (*r_it)->getAttribute("type")->getValue(); // mandatory
          const anna::xml::Attribute *qual = (*r_it)->getAttribute("qual", anna::Exception::Mode::Ignore); // implied
          // Assignment:
          const Avp * avp = getAvp(id);

          if(avp == NULL) {
            std::string s_ex = anna::functions::asString("Avp '%s', referenced at avp rule definition within grouped avp '%s', not found at xml", id.c_str(), gid);
            throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
          }

          auxAvpRule.setAvpId(avp->getId());
          auxAvpRule.setPresence(AvpRule::Presence::asEnum(type));
          auxAvpRule.setQual(qual ? (qual->getValue()) : "");
          gavp->addAvpRule(auxAvpRule);
        }
      } // grouped
    } // end if
  } // end for

  // Check avp loops between grouped avps:

  // In order to avoid loops, we could force to define grouped avps which children
  //  had been previously defined at xml file. In this way, is imposible to get a loop:
  //   C = ...
  //   D = ...
  //   A = grouped of B,C,D -> error, B unknown
  //   B = grouped of A,F -> with former definition, would become a loop
  //
  // But this supposes a restriction at xml configuration (specific order).
  // The other way is an internal check: a grouped AVP won't have descendants within
  //  its ascendants. Then we will check all grouped avps in this way:
  //
  // 1. Searching for another grouped avps which are parents for this avp.
  // 2. If these are children (even this avp(*)) at avp definition, then a loop is detected.
  //
  // Example 1: (1) Analyzing 'A', found parent 'B' / (2) 'B' is already children of 'A'
  //       A -> B
  //            C
  //            D
  //       ...
  //       B -> A -> loop !!
  //            F
  //
  // (*) Example 2: (1) Analyzing 'A', found parent 'A' / (2) 'A' is already children of 'A'
  //       A -> B
  //            C
  //            D
  //            A -> loop !!
  //
  for(const_avp_iterator it = avp_begin(); it != avp_end(); it++) {
    const Avp & avp = (*it).second;

    if(!((avp.getFormat())->isGrouped())) continue;

    for(const_avp_iterator it_p = avp_begin(); it_p != avp_end(); it_p++) {
      const Avp & avp_p = (*it_p).second;

      if(!((avp_p.getFormat())->isGrouped())) continue;

      if(avp_p.isChild(avp.getId())) {
        if(avp.isChild(avp_p.getId())) {
          std::string s_ex;

          if(it != it_p)
            s_ex = anna::functions::asString("Loop detected between grouped avps '%s' and '%s'", avp.getName().c_str(), avp_p.getName().c_str());
          else
            s_ex = anna::functions::asString("Loop within grouped avp '%s': cannot contain itself !!", avp.getName().c_str());

          throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
        } // parent is children of (ref): loop !
      } // parent found
    } // search parents
  } // search grouped avps (ref)
}


//------------------------------------------------------------------------------
//------------------------------------------------ Dictionary::extractCommands()
//------------------------------------------------------------------------------
void Dictionary::extractCommands(const anna::xml::Node *rootNode) throw(anna::RuntimeException) {
  Command auxCommand;

  // (*) Avp rules adding:
  for(anna::xml::Node::const_child_iterator it = rootNode->child_begin(); it != rootNode->child_end(); it++) {
    std::string nodeName = (*it)->getName();

    if(nodeName == "command") {
      // Reset:
      auxCommand.initialize(this);
      // Attributes:
      const char * name = (*it)->getAttribute("name")->getCStringValue(); // mandatory
      U24 code = (*it)->getAttribute("code")->getIntegerValue(); // mandatory
      std::string type = (*it)->getAttribute("type")->getValue(); // mandatory
      // Assignment:
      auxCommand.setCode(code);
      auxCommand.setName(name);
      auxCommand.setRequest(type == "Request");
      AvpRule auxAvpRule(this);

      for(anna::xml::Node::const_child_iterator r_it = (*it)->child_begin(); r_it != (*it)->child_end(); r_it++) {
        // Attributes:
        std::string id = (*r_it)->getAttribute("id")->getValue(); // mandatory
        std::string type = (*r_it)->getAttribute("type")->getValue(); // mandatory
        const anna::xml::Attribute *qual = (*r_it)->getAttribute("qual", anna::Exception::Mode::Ignore); // implied
        // Assignment:
        const Avp * avp = getAvp(id);

        if(avp == NULL) {
          std::string s_ex = anna::functions::asString("Avp '%s', referenced at avp rule definition within command '%s', not found at xml", id.c_str(), name);
          throw anna::RuntimeException(s_ex, ANNA_FILE_LOCATION);
        }

        auxAvpRule.setAvpId(avp->getId());
        auxAvpRule.setPresence(AvpRule::Presence::asEnum(type));
        auxAvpRule.setQual(qual ? (qual->getValue()) : "");
        auxCommand.addAvpRule(auxAvpRule);
      }

      // New entry:
      addCommand(auxCommand);
    } // end if
  } // end for
}


//------------------------------------------------------------------------------
//----------------------------------------------------------- Dictionary::load()
//------------------------------------------------------------------------------
void Dictionary::load(const std::string & pathFile) throw(anna::RuntimeException) {
  if(pathFile == "")
    throw anna::RuntimeException("Empty xml path file provided", ANNA_FILE_LOCATION);

  LOGDEBUG(
    std::string trace = "Loading diameter dictionary from '";
    trace += pathFile;
    trace += "' ...";
    anna::Logger::debug(trace, ANNA_FILE_LOCATION);
  );

  try {
    anna::xml::DocumentFile xmlDocument; // has private copy constructor defined but not implemented to avoid inhenrit/copy (is very heavy)
    const anna::xml::Node *rootNode;
    xmlDocument.initialize(pathFile.c_str()); // fail here is i/o error
    rootNode = xmlDocument.parse(*a_dtd); // Parsing: fail here if xml violates dtd
    a_name = rootNode->getAttribute("name")->getValue();
    //checkUniqueIdentifiers(rootNode);   // Check unique id within xml, for vendor, avp and command nodes:
    extractFormats(rootNode);           // Add formats:
    extractVendors(rootNode);           // Add vendors:
    extractAvps(rootNode);              // Add avps:
    extractCommands(rootNode);          // Add commands:
  } catch(anna::RuntimeException& ex) {
    ////ex.trace();
    throw;
  }

  // Trace:
  LOGDEBUG(anna::Logger::debug(asString(), ANNA_FILE_LOCATION));
}

