// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/mt/Guard.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/util/DelayMeter.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/app/Application.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/CongestionController.hpp>

#include <anna/test/Control.hpp>

using namespace std;
using namespace anna;

typedef Guard MyGuard;
//typedef Guard <test::Control> MyGuard;

test::Control::Control (comm::Communicator* engine) :
   a_engine (*engine),
   a_maxMessage (-1),
   a_initTime (0),
   a_avgDelay ("AvgDelay"),
   a_avgLatency ("AvgLatency"),
   a_delay (0)
{;}

bool test::Control::canContinue (const comm::socket::Client&)
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod ttmm (Logger::Local7, "test::Control::canContinue", ANNA_FILE_LOCATION));

   comm::CongestionController& congestionController = comm::CongestionController::instantiate ();

   if (a_maxMessage != -1 && congestionController.getMessageCounter () >= a_maxMessage) {
      stop ();
      return false;
   }

   MyGuard guard (this);

   if (a_initTime == 0)
      a_initTime = anna::Millisecond::getTime ();

   return true;
}

void test::Control::delay ()
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod ttmm (Logger::Local7, "test::Control::delay", ANNA_FILE_LOCATION));

  if (a_delay > 0) {
      Millisecond random = Millisecond ((a_delay > 10) ? (rand () % (a_delay / 10)): 0);
      Millisecond delay (a_delay);

      int sign = rand () % 2;

      if (sign == 0)
         delay -= random;
      else
         delay += random;

      DelayMeter <Microsecond> meter;

      anna::functions::sleep (delay);

      if (true) {
         MyGuard guard (this);
         a_avgDelay += meter.getValue();
      }
   }
}

// Aproximacin del valor que lleva en mensaje en cola I/O
Millisecond test::Control::latency (const Millisecond& init)
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod ttmm (Logger::Local7, "test::Control::latency", ANNA_FILE_LOCATION));

   const Millisecond now = anna::Millisecond::getTime();

   const Millisecond _latency = now - init;

   if (_latency > 0) {
      MyGuard guard (this);
      a_avgLatency += _latency;
   }

   return now;
}

void  test::Control::report ()
   throw ()
{
   const Millisecond serviceTime = anna::Millisecond::getTime () - a_initTime;

   comm::CongestionController& congestionController = comm::CongestionController::instantiate ();

   const int workload = (serviceTime == 0) ? 0: congestionController.getMessageCounter () * 1000 / serviceTime;
   string msg (anna::functions::asText ("Time of service: ", (int) serviceTime));
   msg += anna::functions::asText (" ms | Workload: ", workload);
   msg += anna::functions::asText (" msg/sec | Received Messages: ", congestionController.getMessageCounter ());
   msg += anna::functions::asText (" | Processed Messages: ", congestionController.getSuccessCounter ());
   Logger::notice (msg, ANNA_FILE_LOCATION);
   cout << msg << endl << endl;

   msg = "Average delay (us): ";
   msg += a_avgDelay.asString ();
   Logger::notice (msg, ANNA_FILE_LOCATION);
   cout << msg << endl << endl;

   msg = "Average latency (ms): ";
   msg += a_avgLatency.asString ();
   Logger::notice (msg, ANNA_FILE_LOCATION);
   cout << msg << endl << endl;
}

void test::Control::stop ()
   throw (RuntimeException)
{
   if (a_engine.hasRequestedStop () == true)
      return;

   a_engine.requestStop ();

}

xml::Node* test::Control::asXML (xml::Node* parent)
   throw ()
{
   xml::Node* result = parent->createChild ("test.Control");

   result->createAttribute ("MaxMessage", a_maxMessage);
   result->createAttribute ("AvgDelay", a_avgDelay.value ().asString());
   result->createAttribute ("AvgLatency", a_avgLatency.value ().asString());

   return result;
}

